###############################################################################################
# $Id: action.py,v 1.1 2002/02/13 11:10:38 chakie Exp $
###############################################################################################

class Action:
    """
    This class is a baseclass for all available types of actions that can be sent from the server to
    the clients. Actions are used to send information about everything that has happened in a turn
    to the clients. Each action instance contains some small update to a unit or some other part of
    the client's datastructures.

    Actions are meant to be serialized to the remote client and 'given' to the local client. In the
    action phase the clients will apply all action for a given step, and let the players see the
    effects. 
    
    Every action contains a name which is a short lowercase string that identifies it. The name is
    unique only among classes, not instances. The name can be used to recognize the update when in
    deep debug mode.
    """
    
    def __init__ (self, name, step):
        """Initializes the instance. Stores the name."""
        # store the name
        self.name = name

        # store the step too
        self.step = step


    def getName (self):
        """Returns the name of the action. This is a string that can be used to identify the class."""
        return self.name


    def getStep (self):
        """Returns the step of the action phase when this instance should be applied."""
        return self.step


    def extract (self, parameters):
        """Extracts all data from the data coming from the network. This method is used whenever
        the instance has been written to the socket and an instance needs to be recreated from textual
        data.  This method should be overridden to perform whatever is needed. The 'parameters' is a
        list of all the parameters passed with the packet."""
        pass
    

    def execute (self):
        """This method applies the action to the client data structures. Usually it means changing
        some data for one a unit. It must be overridden by subclasses."""
        
        raise NotImplementedError ( "Action.execute: this method must be overridden")

 
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
