###############################################################################################
# $Id: add_plan_act.py,v 1.4 2002/09/08 11:26:57 chakie Exp $
###############################################################################################

import scenario
from action      import Action
import plan.factory


## def create ():
##     """This function is a factory for creating instances of the class 'AddPlanAct'. Use this when you need
##     to create instances. The new instance is returned."""
##     # create a new instance and return it
##     return AddPlanAct ()


class AddPlanAct (Action):
    """
    This class implements the immediate update 'add plan'. This is sent by the server when an unit
    has finished a plan and it should be removed from the unit's plans. The first plan that the unit
    has (the one that's supposed to be active) is removed."""

    def __init__ (self, step=-1, unitid=-1, plan=None ):
        """Initializes the instance."""
        # call superclass constructor
        Action.__init__ (self, "add_plan_act", step )

        # store all data
        self.unitid = unitid

        # store the plan
        self.plan = plan
        

    def extract (self, parameters):
        """Extracts the data for the command. """
        # parse out the data
        self.step   = int ( parameters[0] )
        self.unitid = int ( parameters[1] )

        print "AddPlanAct.extract:", parameters [2:]

        # create the plan from the rest of the data
        self.plan = plan.factory.create ( parameters [2:] )
        
        
    def execute (self):
        """Executed the action. Finds the affected unit and just removes the first plan it has."""
        
        # get the unit with the given id 
        unit = scenario.info.units [ self.unitid ]

        # is it a local unit? only do this to local units
        if unit.getOwner () == scenario.local_player_id:
            # add the new plan
            unit.getPlans ().append ( self.plan )
        
 
    def toString (self):
        """Returns a string representation of the command, suitable for sending over a socket."""
        # create a string and return
        return "%s %d %d %s\n" % ( self.getName (), self.step, self.unitid,
                                   self.plan.toString ().strip () )


    def __str__ (self):
        """Convenience wrapper for toString() suitable for using when debugging and printing the
        command to the screen. Will just call toString()."""
        return self.toString ()
    
     
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
