###############################################################################################
# $Id: change_modifiers_act.py,v 1.4 2002/09/08 11:26:57 chakie Exp $
###############################################################################################

import scenario
from action      import Action

## def create ():
##     """This function is a factory for creating instances of the class 'ChangeModifiersAct'. Use this when you need
##     to create instances. The new instance is returned."""
##     # create a new instance and return it
##     return ChangeModifiersAct ()


class ChangeModifiersAct (Action):
    """
    This class implements the action 'change_modifier'. This is sent by the server when an unit
    has got one or more of its modifiers changed. All of them need not have changed, but they are
    all transmitted in this action anyway. The sent ones are:

    * fatigue
    * morale
    * experience

    The data is simply just applied to the concerned unit.
    """

    def __init__ (self, step=-1, unitid=-1, fatigue=-1, morale=-1, experience=-1):
        """Initializes the instance."""
        # call superclass constructor
        Action.__init__ (self, "change_modifiers_act", step )

        # store all data
        self.unitid     = unitid
        self.fatigue    = fatigue
        self.morale     = morale
        self.experience = experience


    def extract (self, parameters):
        """Extracts the data for the command. """
        # parse out the data
        self.step       = int ( parameters[0] )
        self.unitid     = int ( parameters[1] )
        self.fatigue    = int ( parameters[2] )
        self.morale     = int ( parameters[3] )
        self.experience = int ( parameters[4] )

        
    def execute (self):
        """Executed the action. Finds the affected unit and updates its modifiers."""

        # precautions
        if not scenario.info.units.has_key ( self.unitid ):
            # no such unit? bad stuff...
            raise "could not find unit: %d" % self.unitid
        
        # get the unit with the given id 
        unit = scenario.info.units [ self.unitid ]

        # set the new modifiers
        unit.getFatigue ().setValue    ( self.fatigue )
        unit.getMorale ().setValue     ( self.morale )
        unit.getExperience ().setValue ( self.experience )
          
        # make sure the world knows of this change
        if not scenario.local_player_ai:
            scenario.dispatcher.emit ( 'units_changed', (unit,) )
        
 
    def toString (self):
        """Returns a string representation of the command, suitable for sending over a socket."""
        # create a string and return
        return "%s %d %d %d %d %d\n" % ( self.getName (), self.step, self.unitid, self.fatigue,
                                         self.morale, self.experience )


    def __str__ (self):
        """Convenience wrapper for toString() suitable for using when debugging and printing the
        command to the screen. Will just call toString()."""
        return self.toString ()
    
     
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
