###############################################################################################
# $Id: damage_act.py,v 1.15 2002/09/08 11:26:57 chakie Exp $
###############################################################################################

import scenario
from action                   import Action
from messages                 import COMBAT, DESTROYED

# define the possible types of damage
KILLED        = 0
GUNSDESTROYED = 1

## def create ():
##     """This function is a factory for creating instances of the class 'DamageAct'. Use this when you need
##     to create instances. The new instance is returned."""
##     # create a new instance and return it
##     return DamageAct ()


class DamageAct (Action):

    """This class implements the immediate update 'damage'. This command is sent when a unit has
    been shot at. This update contains the id of the affected unit, the type of damage and the
    amount of that particular damage. The type can be lost guns and men, and the amount thus the
    number of guns/men.

    If the unit is destroyed it will be
    """

    def __init__ (self, step=-1, unitid=-1, damagetype=-1, damagecount=-1):
        """Initializes the instance."""
        # call superclass constructor
        Action.__init__ (self, "damage_act", step )

        # set illegal values for all data
        self.unitid      = unitid
        self.damagetype  = damagetype
        self.damagecount = damagecount


    def extract (self, parameters):
        """Extracts the data for the move."""

        # parse out the data
        self.step        = int ( parameters[0] )
        self.unitid      = int ( parameters[1] )
        self.damagetype  = int ( parameters[2] )
        self.damagecount = int ( parameters[3] )

        
    def execute (self):
        """Executed the command. Finds the affected unit and updates the damage. If the unit is
        destroyed it will be removed and all units targetting it will lose their targets."""
 
        # get the unit with the given id
        unit = scenario.info.units [ self.unitid ]

        # apply the damage. if the unit is destroyed it will mark itself as destroyed and remove
        # itself from the global unit structures
        unit.applyDamage (self.damagetype, self.damagecount)

        # is the unit destroyed? we'll let the world know of this, but not for the ai
        if not scenario.local_player_ai:
            if unit.isDestroyed ():
                # let the world know of the destroyed unit
                scenario.dispatcher.emit ( 'unit_destroyed', (unit,) )
            else:
                # not destroyed, just changed 
                scenario.dispatcher.emit ( 'units_changed', (unit,) )

        # do we need to show a message in the messages? only for local players
        if self.damagecount > 0 and unit.getOwner () == scenario.local_player_id:
            # what type of damage?
            if self.damagetype == KILLED:
                # lost men
                message = '%s lost %d men' % (unit.getName (), self.damagecount)
            else:
                # lost guns
                message = '%s lost %d guns' % (unit.getName (), self.damagecount)

            # add the message if suitable
            if not scenario.local_player_ai:
                scenario.messages.add ( message, COMBAT )

        # did it get destroyed too?
        if unit.destroyed == 1:
            # no men left, it's a goner
            message = '%s is destroyed!' % unit.getName () 

            # add the message if suitable
            if not scenario.local_player_ai:
                scenario.messages.add ( message, DESTROYED )

            # clear units that targetted the destroyed one
            self.clearTargetters ( unit )

            # was the unit selected?
            if unit in scenario.selected:
                # delete it
                scenario.selected.remove ( unit )

            # let the world know that this unit should no longer be selected
            if not scenario.local_player_ai:
                scenario.dispatcher.emit ( 'unit_selected', None )


    def clearTargetters (self, destroyed):
        """Makes sure no unit it targetting this unit anymore. This is used when the unit has been
        destroyed. Loops through all units and clears their target if they were targetting this
        unit."""
        
        # find all units (enemies) who have targeted this unit
        for unit in scenario.info.units.values ():
            # did it target this unit?
            if unit.getTarget () == destroyed:
                # yes, clear the target
                unit.setTarget ( None )

                #print "DamageAct.clearTargetters: cleared: %d, %s" % ( unit.getId(), unit.getName())


    def toString (self):
        """Returns a string representation of the command, suitable for sending over a socket."""
        # create a string and return
        return "%s %d %d %d %d\n" % ( self.getName (), self.step, self.unitid, self.damagetype,
                                      self.damagecount )


    def __str__ (self):
        """Convenience wrapper for toString() suitable for using when debugging and printing the
        command to the screen. Will just call toString()."""
        return self.toString ()
 
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
