###############################################################################################
# $Id: move_act.py,v 1.7 2002/09/08 11:26:57 chakie Exp $
###############################################################################################

import scenario
from action      import Action

## def create ():
##     """This function is a factory for creating instances of the class 'MoveAct'. Use this when you need
##     to create instances. The new instance is returned."""
##     # create a new instance and return it
##     return MoveAct ()


class MoveAct (Action):
    """
    This class implements the immediate update 'move'. This is sent by the server when an unit
    should move to a new position. Facing, mode or other data is not affected in any way, just the
    unit position.
    """

    def __init__ (self, step=-1, unitid=-1, x=-1, y=-1 ):
        """Initializes the instance."""
        # call superclass constructor
        Action.__init__ (self, "move_act", step )

        # store all data
        self.unitid = unitid
        self.x      = x
        self.y      = y
        

    def extract (self, parameters):
        """Extracts the data for the command. """
        # parse out the data
        self.step   = int ( parameters[0] )
        self.unitid = int ( parameters[1] )
        self.x      = int ( parameters[2] )
        self.y      = int ( parameters[3] )

        
    def execute (self):
        """Executed the action. Finds the affected unit and updates its position to the new
        position."""
               
        # get the unit with the given id 
        unit = scenario.info.units [ self.unitid ]

        # set the new position
        unit.setPosition ( (self.x, self.y) )

        # recalculate the units the unit sees.
        # TODO: this is a fairly heavy operation, should it really be done all the time?
        unit.calcView ()
          
        # make sure the world knows of this change
        if not scenario.local_player_ai:
            scenario.dispatcher.emit ( 'units_changed', (unit,) )
        
 
    def toString (self):
        """Returns a string representation of the command, suitable for sending over a socket."""
        # create a string and return
        return "%s %d %d %d %d\n" % ( self.getName (), self.step, self.unitid, self.x, self.y )


    def __str__ (self):
        """Convenience wrapper for toString() suitable for using when debugging and printing the
        command to the screen. Will just call toString()."""
        return self.toString ()
    
     
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
