###############################################################################################
# $Id: rotate_act.py,v 1.5 2002/09/08 11:26:57 chakie Exp $
###############################################################################################

import scenario
from action      import Action

## def create ():
##     """This function is a factory for creating instances of the class 'RotateAct'. Use this when you need
##     to create instances. The new instance is returned."""
##     # create a new instance and return it
##     return RotateAct ()


class RotateAct (Action):
    """
    
    This class implements the immediate update 'endgame'. This is sent by the server when the game
    has been ended for some reason. The reason could be that one player has lost, resigned or that
    the time allocated for the battle has run out. This update will set a flag that terminates the
    main event loops for the client and the AI players.

    This command also sets a flag that indicates how the game ended, i.e. which player won, was it a
    surrender etc. The costants are defined in the file constants.py.
    """

    def __init__ (self, step=-1, unitid=-1, facing=-1 ):
        """Initializes the instance."""
        # call superclass constructor
        Action.__init__ (self, "rotate_act", step )

        # store all data
        self.unitid = unitid
        self.facing = facing
        

    def extract (self, parameters):
        """Extracts the data for the command. """
        # parse out the data
        self.step   = int ( parameters[0] )
        self.unitid = int ( parameters[1] )
        self.facing = int ( parameters[2] )

        
    def execute (self):
        """Executed the command. Finds the affected unit and updates its facing to the new
        facing."""
               
        # get the unit with the given id 
        unit = scenario.info.units [ self.unitid ]

        # set the new facing
        unit.setFacing ( self.facing )
           
        # make sure the world knows of this change
        if not scenario.local_player_ai:
            scenario.dispatcher.emit ( 'units_changed', (unit,) )
       
 
    def toString (self):
        """Returns a string representation of the command, suitable for sending over a socket."""
        # create a string and return
        return "%s %d %d %d\n" % ( self.getName (), self.step, self.unitid, self.facing )


    def __str__ (self):
        """Convenience wrapper for toString() suitable for using when debugging and printing the
        command to the screen. Will just call toString()."""
        return self.toString ()
    
     
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
