###############################################################################################
# $Id: set_target_act.py,v 1.7 2002/09/08 11:26:57 chakie Exp $
###############################################################################################
import pygame
from pygame.locals           import *

import scenario
from action                  import Action

from mode.mode_factory       import createMode


## def create ():
##     """This function is a factory for creating instances of the class 'SetTargetAct'. Use this when you need
##     to create instances. The new instance is returned."""
##     # create a new instance
##     return SetTargetAct ()


class SetTargetAct (Action):
    """
    This class implements the immediate update 'set target'. This is sent by the server when an unit
    gets a new target it is firing at. IT can also be used to clear an old target, which is done
    when giving a target id of -1.
    
    """

    def __init__ (self, step=-1, unitid=-1, targetid=-1):
        """Initializes the instance."""
        # call superclass constructor, the turn may not be valid here yet
        Action.__init__ (self, "settarget_act", step )

        # set values for all data
        self.unitid = unitid
        self.targetid = targetid


    def extract (self, parameters):
        """Extracts the data for the move."""

        # parse out the data
        self.step     = int ( parameters[0] )
        self.unitid   = int ( parameters[1] )
        self.targetid = int ( parameters[2] )

 
    def execute (self):
        """Executed the action. Finds the affected unit and updates its target to the new target."""
        
        # get the unit that is setting the target
        unit = scenario.info.units [ self.unitid ]

        # do we have a target too? or are we clearing an old target?
        if self.targetid == -1:
            # clearing an old target
            target = None

        else:
            # setting a new target
            target = scenario.info.units [ self.targetid ]

        # and assign the target
        unit.setTarget ( target )
          
        # make sure the world knows of this change
        if not scenario.local_player_ai:
            scenario.dispatcher.emit ( 'units_changed', (unit,) )
        
 
    def toString (self):
        """Returns a string representation of the command, suitable for sending over a socket."""
        # create a string
        return "%s %d %d %d\n" % ( self.getName (), self.step, self.unitid, self.targetid )
        

    def __str__ (self):
        """Convenience wrapper for toString() suitable for using when debugging and printing the
        command to the screen. Will just call toString()."""
        return self.toString ()

 
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
