#! /usr/bin/env python2
###############################################################################################
# $Id: civil-scenario-server.py,v 1.1 2001/10/15 15:30:37 chakie Exp $
###############################################################################################
#
# This application is a sample test application that works as a scenario repository server. It can
# serve out scenarios using XML-RPC to clients on the 'net. The clients can get information such as
# the number of available scenarios, a list of scenarios, scenario names and of course full
# scenarios.
#
# The server reads the scenarios from a directory which is given as a commandline parameter. The
# directory must contain a 'scenarioindex.xml' file which contains condensed info about the
# scenarios in the directory.
#
###############################################################################################

import sys
import os.path
import SocketServer

from scenario_manager                import ServerScenarioManager
from scenario_server.scenario_server import ScenarioServer


def readScenarios (directory):
    """Reads in the list of scenarios from file. Returns a created ServerScenarioManager that
    contains data about all the classes"""

    # does the index file exist?
    if not os.path.exists ( directory + '/scenarioindex.xml' ):
        print "could not find index file 'scenarioindex.xml' in '%s' " % directory
        print "usage: %s directory" % sys.argv[1]
        sys.exit ( 1 )

    # seems to be ok, create the manager
    manager = ServerScenarioManager ()
    
    # read in all scenario info:s
    if not manager.loadScenarioIndex ( directory + '/scenarioindex.xml' ):
        # loading failed
        print "failed to load index file %s" % ( directory + '/scenarioindex.xml')
        sys.exit ( 1 )

    print "read scenarios from %s" % os.path.abspath (directory + '/scenarioindex.xml')

    return manager


###############################################################################################
if __name__ == '__main__':

    # did we get any commandline parameter? this is required and should be the directory where the
    # scenario data is to be found
    if len ( sys.argv ) != 2:
        print "invalid number of parameters"
        print "usage: %s directory" % ( sys.argv[0] )
        sys.exit ( 1 )

    # is it a directory?
    if not os.path.isdir ( sys.argv[1] ):
        # no a directory
        print "%s is not a directory" % ( sys.argv[1] )
        print "usage: %s directory" % ( sys.argv[1] )
        sys.exit ( 1 )
        
    # read the scenarios from the passed directory
    readScenarios ( sys.argv[1] )

    print "scenario info loaded, waiting for clients on port 8000"
    
    # create the xml-rpc server
    server = SocketServer.TCPServer ( ('', 8000), ScenarioServer )
    
    # and server requests forever
    try:
        server.serve_forever()

    except KeyboardInterrupt:
        # we got killed, go away
        print "terminated by user, exiting"
        sys.exit ( 0 )



#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
