###############################################################################################
# $Id: new_scenario.py,v 1.10 2002/09/05 17:33:06 msa Exp $
###############################################################################################

from qt import *

import scenario
from map           import Map
from scenario_info import ScenarioInfo
from hex           import Hex
from organization  import REBEL, UNION

import random

def createNewScenario(mainwindow, width, height):
    randgen = random.Random()
    # create new scenario info
    scenario.info = ScenarioInfo ()
    
    # set the scenario info
    scenario.info.setName ( 'No name' )
    scenario.info.setDescription ( ['No description'] )
    scenario.info.setMission ( UNION, ['No union mission'] )
    scenario.info.setMission ( REBEL, ['No rebel mission'] )
    scenario.info.setLocation ( 'No location' )
    scenario.info.setMaxTurns ( 30 )
    scenario.info.setDate ( ( 1962, 1, 1, 9, 0 ) )
    
    # create the map
    scenario.map = Map ( width, height )

    print "NewScenario.ok:", scenario.map.getSize ()

    # "Suitable" grass icons
    mapper = [2,2,2,2,2,2,3,3,3,3,5,12]
    
    # loop and set initial icons for the map
    for y in range (height):
        for x in range (width):
            scenario.map.getHexes () [y][x] = Hex ( mapper[ randgen.randrange(0, len(mapper)) ] )


    # Make sure everything knows we have new scenario
    mainwindow.mapview.refresh()
    mainwindow.unionview.refresh()
    mainwindow.rebelview.refresh()
    mainwindow.objectiveview.refresh()
    mainwindow.blockview.refresh()
    #mainwindow.weaponview.refresh()
    mainwindow.scenarioview.refresh()
    
    # enable all tabs
    mainwindow.palette.setEnabled ( 1 )
    



class NewScenario(QDialog):
    """This class is a dialog that takes care of createing a new scenario. It will query the user
    for the size of the map and if accepted initialize a new empty map and reset the scenario info.
    All other scneario data it set afterwards in the special dedicated tab, so data such as the
    scenario name, description etc are given default values here."""

    def __init__(self,parent):
        # create a new modal dialog
        QDialog.__init__(self, parent, 'NewScenario', 1)

        self.setCaption(self.tr('Create a new scenario'))
        self.setSizeGripEnabled(0)
        NewScenarioLayout = QVBoxLayout(self)
        NewScenarioLayout.setSpacing(10)
        NewScenarioLayout.setMargin(11)

        self.TextLabel1 = QLabel(self,'TextLabel1')
        self.TextLabel1.setText(self.tr('Enter the wanted width and height of the new map. All other extra information about the scenario is input on the <b>Scenario data</b> tab.'))
        self.TextLabel1.setAlignment(QLabel.WordBreak | QLabel.AlignVCenter | QLabel.AlignLeft)
        NewScenarioLayout.addWidget(self.TextLabel1)

        Layout3 = QGridLayout()
        Layout3.setSpacing(10)
        Layout3.setMargin(0)

        self.TextLabel2 = QLabel(self,'TextLabel2')
        self.TextLabel2.setSizePolicy(QSizePolicy(3,1,self.TextLabel2.sizePolicy().hasHeightForWidth()))
        self.TextLabel2.setText(self.tr('Width:'))

        Layout3.addWidget(self.TextLabel2,0,0)
        spacer = QSpacerItem(20,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        Layout3.addItem(spacer,0,2)

        self.width = QSpinBox(self,'width')
        self.width.setSizePolicy(QSizePolicy(3,0,self.width.sizePolicy().hasHeightForWidth()))
        self.width.setMaxValue(500)
        self.width.setMinValue(10)
        self.width.setValue(40)

        Layout3.addWidget(self.width,0,1)
        spacer_2 = QSpacerItem(20,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        Layout3.addItem(spacer_2,1,2)

        self.height = QSpinBox(self,'height')
        self.height.setSizePolicy(QSizePolicy(3,0,self.height.sizePolicy().hasHeightForWidth()))
        self.height.setMaxValue(500)
        self.height.setMinValue(10)
        self.height.setValue(40)

        Layout3.addWidget(self.height,1,1)

        self.TextLabel3 = QLabel(self,'TextLabel3')
        self.TextLabel3.setSizePolicy(QSizePolicy(3,1,self.TextLabel3.sizePolicy().hasHeightForWidth()))
        self.TextLabel3.setText(self.tr('Height:'))

        Layout3.addWidget(self.TextLabel3,1,0)
        NewScenarioLayout.addLayout(Layout3)
        spacer_3 = QSpacerItem(20,20,QSizePolicy.Minimum,QSizePolicy.Expanding)
        NewScenarioLayout.addItem(spacer_3)

        Layout4 = QHBoxLayout()
        Layout4.setSpacing(10)
        Layout4.setMargin(0)
        spacer_4 = QSpacerItem(20,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        Layout4.addItem(spacer_4)

        self.btnok = QPushButton(self,'btnok')
        self.btnok.setText(self.tr('&Ok'))
        self.btnok.setDefault(1)
        Layout4.addWidget(self.btnok)

        self.btncancel = QPushButton(self,'btncancel')
        self.btncancel.setText(self.tr('&Cancel'))
        Layout4.addWidget(self.btncancel)
        NewScenarioLayout.addLayout(Layout4)

        self.connect(self.btnok,SIGNAL('clicked()'),self.ok)
        self.connect(self.btncancel,SIGNAL('clicked()'),self.cancel)

        self.resize( self.sizeHint() )
        

    def cancel(self):
        """Callback triggered when the Cancel button is pressed. Simply rejects the dialog without
        any further actions. """ 

        # reject the dialog
        self.reject ()


    def ok(self):
        """Callback triggered when the Ok button is pressed. Accepts the dialog and creates a new
        map and resets all scenario info."""

        createNewScenario(self.parent(), self.width.value(), self.height.value())

        # accept ourselves
        self.accept ()
   
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:

