###############################################################################################
# $Id: pluginview.py,v 1.5 2002/07/25 16:03:30 msa Exp $
###############################################################################################

from qt import *
import os
import sys
import glob
import plugins.plugin
import stat

class PluginView(QListBox):
    """This class is a simple view that is shown in the main tab bar. It contains a list of all the
    plugins that are available. It lets the user select a plugin and then start doing stuff in the
    map. It mainly relays events to the currently selected plugin."""

    def __init__(self, mainwindow):
        """Goes trough the plugins directory, searching for all plugins"""
        QListBox.__init__(self, mainwindow)
        self.mainwindow = mainwindow

        self.plugins = []
        self.findPlugins()


        # we want single selection
        self.setSelectionMode ( QListBox.Single )

    def findPlugins(self):
        for file in glob.glob('editor/plugins/*.py'): # BUG directory separator depending on OS?
            mode = os.stat(file)[0]
            if not stat.S_ISREG(mode):
                continue

            file = file[:-3] # skip .py

            plugin = __import__(file, globals(), locals(), file)
            if not plugin.__dict__.has_key("new"):
                continue
            instance = plugin.new(self.mainwindow.iconview)
            if not instance:
                continue
            item = QString(instance.name)
            self.insertItem(item)
            self.plugins.append(instance)
            print "Added plugin %s..." % instance.name

    def getPlugin(self):
        # get the selected plugin index (if any)
        index = self.currentItem ()

        # do we have a selected plugin?
        if index == -1:
            # nope, go away
            return None

        return self.plugins[index]
            
    def mapClickedLeft (self, x, y, hexx, hexy):
        """Callback triggered when the map has been clicked with the left mouse button. Relays the
        event to the currently selected plugin, if any."""

        plugin = self.getPlugin()
        if not plugin:
            return

        plugin.mapClickedLeft(x,y,hexx, hexy)
        
        print "PluginView.mapClickedLeft"
  

    def mapClickedMid (self, x, y, hexx, hexy):
        """Callback triggered when the map has been clicked with the middle mouse button. Relays the
        event to the currently selected plugin, if any.""" 

        plugin = self.getPlugin()
        if not plugin:
            return

        plugin.mapClickedMid(x,y,hexx, hexy)

        print "PluginView.mapClickedMid"


    def mapClickedRight (self, x, y, hexx, hexy):
        """Callback triggered when the map has been clicked with the right mouse button. Relays the
        event to the currently selected plugin, if any.""" 

        plugin = self.getPlugin()
        if not plugin:
            return

        plugin.mapClickedRight(x,y,hexx, hexy)

        print "PluginView.mapClickedRight"

    
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:

