###############################################################################################
# $Id: rally.py,v 1.5 2002/09/04 16:44:34 msa Exp $
###############################################################################################

import scenario

from engine.ai.module       import Module
from action.set_target_act  import SetTargetAct


class Rally (Module):
    """This class defines a module that 

    This module is executed every step.
    """


    def __init__ (self):
        """Initializes the module. Sets a suitable name."""

        # call superclass
        Module.__init__ (self, 'rally' )


    def isReady (self, step):
        """Checks wether the module is ready to be executed. This module is executed every 10th
        step, ie. when step % 10 = 0."""
        # has 10 steps passed?
        if step % 10 == 0:
            # yep, we're ready
            return 1

        # not yet
        return 0
    

    def execute (self, step):
        """Executes the module. Loops over all units and checks wether they are disorganized or
        routed. Checks wether the unit can rally and if it can then it's mode is set to the mode it
        should have after after being disorganized/routed.

        A unit can rally or become disorganized if its morale is high enough. The limits for
        rallying are:

        * routed -> disorganized: 30
        * disorganized -> normal: 45
        """

        # no action yet
        actiondata = []

        # TODO: using hardcoded morale values here
        
        # loop over all units we have
        for unit in scenario.info.units.values ():
            # is the unit currently able to rally?
            if unit.getMode ().canRally ():
                # yes, it could optionally rally, it its morale high enough?
                if unit.getMorale ().getValue () > 45:
                    # yep, it can rally
                    self.setMode ( unit, unit.getMode ().onRally (), actiondata, step )

            # no, it can't rally, can it become disorganized, ie is it routed?
            elif unit.getMode ().canDisorganize ():
                # yes, it could optionally become disorganized, it its morale high enough?
                if unit.getMorale ().getValue () > 30:
                    # yep, it can rally
                    self.setMode ( unit, unit.getMode ().onDisorganize (), actiondata, step )
 

        # we're done with the clearing of targets, return all action we got
        return actiondata

#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
