###############################################################################################
# $Id: button.py,v 1.1 2002/01/22 12:50:10 chakie Exp $
###############################################################################################

from widget import Widget
import widget
import properties
import scenario
import pygame
import pygame.image
import pygame.mixer
from pygame.locals import *

class Button(Widget):
    """
    This class defines a simple pushbutton which can can be clicked. It needs a surfaces that
    represent the button. The surface will be loaded by the constructor from a passed filename.

    This class plays a small sound when the button is clicked.
    """

    # define a shared cache
    cache = {}

    def __init__ (self, filename1, filename2, position = (0,0), callbacks = None):
        """Initializes the widget. Loads the icons from the passed filename."""

        # firts call superclass constructor
        Widget.__init__ (self, position, callbacks)

        # do we have the wanted image in our cache?
        if Button.cache.has_key ( filename1 ):
            # yep, use that instead
            self.surface_released = Button.cache [filename1]
            
        else:
            # not in memory, try to load the surfaces
            self.surface_released = pygame.image.load ( filename1 ).convert ()

            # store in the cache for later use
            Button.cache [filename1] = self.surface_released

        # do we have the wanted image in our cache?
        if Button.cache.has_key ( filename2 ):
            # yep, use that instead
            self.surface_pressed = Button.cache [filename2]
           
        else:
            # not in memory, try to load the surfaces
            self.surface_pressed = pygame.image.load ( filename2 ).convert ()

            # store in the cache for later use
            Button.cache [filename2] = self.surface_pressed

        # initial surface is the non-pressed one
        self.surface = self.surface_released
 
        # set our internal callbacks so that we can trap keys
        self.internal = {widget.MOUSEBUTTONUP   : self.mouseUp,
                         widget.MOUSEBUTTONDOWN : self.mouseDown }


    def mouseUp (self, event):
        """Internal callback triggered when the mouse is released when it is over a button. This
        sets a new icon for the button."""
        # set the new icon
        self.surface = self.surface_released

        # play a sound
        scenario.audio.playSample ( 'button-clicked' )
        
        # we're dirty
        self.dirty = 1


    def mouseDown (self, event):
        """Internal callback triggered when the mouse is pressed when it is over a button. This
        sets a new icon for the button."""
        # set the new icon
        self.surface = self.surface_pressed

        # we're dirty
        self.dirty = 1

        
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
