###############################################################################################
# $Id: setup_client.py,v 1.11 2002/09/05 13:24:56 chakie Exp $
###############################################################################################

import pygame
import pygame.display
import pygame.surface
import getpass
import traceback
import re
import scenario
import properties
import time
from socket         import *
import gui.widget
from gui.image          import Image
from gui.dialog         import *
from gui.button         import Button
from gui.checkbox       import CheckBox
from gui.label          import Label
from gui.editfield      import EditField
from gui.widget_manager import WidgetManager
from gui.messagebox     import Messagebox
from gui.credits        import Credits
from gui.lounge         import Lounge
from gui.lounge_setup   import LoungeSetup


# return codes used to indicate what should happen
CANCEL = 0
OK     = 1

class SetupClient(Dialog):
    """
    This class is used as a dialog for letting the player select wether to run as a client or a
    server. A name can also be given by the players. This is a human readable name that is used for
    the communication between the players. 
    """

    def __init__ (self):
        "Creates the dialog."
        # load the fonts too
        self.titlefont = pygame.font.Font  ( properties.titlefont, 24 )
        self.smallfont = pygame.font.Font  ( properties.textfont, 14 )

        # init superclass
        Dialog.__init__ (self, scenario.sdl)

        # set our background to a tiled image
        self.setBackground ( properties.window_background )

        # not fullscreen by default
        self.use_fullscreen = 0
        

    def createWidgets (self):
        "Creates all widgets for the dialog."
        # labels
        self.wm.register ( Label (self.titlefont, "Setup network information", (20, 10), color=(255, 0, 0)))

        self.wm.register ( Label (self.smallfont, "My name: ",       (300, 190), color=(255, 255, 255)) )
        #self.wm.register ( Label (self.smallfont, "Run as server: ", (250, 300), color=(255, 255, 255)) )

        # get the user name from the environment if it works
        try:
            name = getpass.getuser()
            try:
                import pwd
                # Try to get the user's first name, as given
                # in the password database
                name = pwd.getpwnam(name)[4].split()[0]
            except:
                # Ignore, and use username
                pass
        except:
            name = "My name"

        self.use_this_name = name
        
        # the editfields
        self.user = EditField ( self.smallfont, name,    200, (450, 180), color=(255,255,0),
                                background = (80,80,80))

        self.server = CheckBox ( "Play against a human player, act as server",
                                 self.smallfont,
                                 properties.path_dialogs + "butt-radio-set.png",
                                 properties.path_dialogs + "butt-radio-unset.png",
                                 checked=0, position=(300, 270), color=(255,255,0),
                                 callbacks={widget.MOUSEBUTTONUP : self.toggleServer } )
        self.client = CheckBox ( "Play against a human player, act as client",
                                 self.smallfont,
                                 properties.path_dialogs + "butt-radio-set.png",
                                 properties.path_dialogs + "butt-radio-unset.png",
                                 checked=0, position=(300, 310), color=(255,255,0),
                                 callbacks={widget.MOUSEBUTTONUP : self.toggleServer } )
        self.computer = CheckBox ( "Play against the computer",
                                   self.smallfont,
                                   properties.path_dialogs + "butt-radio-set.png",
                                   properties.path_dialogs + "butt-radio-unset.png",
                                   checked=1, position=(300, 350), color=(255,255,0),
                                   callbacks={widget.MOUSEBUTTONUP : self.toggleServer } )
        self.fullscreen = CheckBox ( "Toggle fullscreen mode",
                                     self.smallfont,
                                     properties.path_dialogs + "butt-radio-set.png",
                                     properties.path_dialogs + "butt-radio-unset.png",
                                     checked=0, position=(300, 430), color=(255,255,0),
                                     callbacks={widget.MOUSEBUTTONUP : self.toggleFullscreen } )
        
        self.wm.register ( self.user )
        self.wm.register ( self.server )
        self.wm.register ( self.client )
        self.wm.register ( self.computer )
        self.wm.register ( self.fullscreen )

        # buttons
        self.wm.register ( Button ( properties.path_dialogs + "butt-ok-moff.png",
                                    properties.path_dialogs + "butt-ok-mover.png",
                                    (40, 650 ), {widget.MOUSEBUTTONUP : self.ok } ) )
        self.wm.register ( Button ( properties.path_dialogs + "butt-cancel-moff.png",
                                    properties.path_dialogs + "butt-cancel-mover.png",
                                    (282, 650 ), {widget.MOUSEBUTTONUP : self.cancel } ) )
        self.wm.register ( Button ( properties.path_dialogs + "butt-credits-moff.png",
                                    properties.path_dialogs + "butt-credits-mover.png",
                                    (528, 650 ), {widget.MOUSEBUTTONUP : self.credits } ) )
        self.wm.register ( Button ( properties.path_dialogs + "butt-lounge-moff.png",
                                    properties.path_dialogs + "butt-lounge-mover.png",
                                    (772, 650 ), {widget.MOUSEBUTTONUP : self.lounge } ) )

        if scenario.commandline_quickstart != 0:
            self.enableTimer(500)
            self.quickstart_state = 0


    def timer(self):
        if 0:
            pass

        elif self.quickstart_state == 0:
            pygame.event.post(pygame.event.Event(widget.MOUSEBUTTONDOWN, { "pos": (320, 370) }))

        elif self.quickstart_state == 1:
            pygame.event.post(pygame.event.Event(widget.MOUSEBUTTONUP,   { "pos": (320, 370) }))

        elif self.quickstart_state == 2:
            pygame.event.post(pygame.event.Event(widget.MOUSEBUTTONDOWN, { "pos": (50, 660) }))

        elif self.quickstart_state == 3:
            pygame.event.post(pygame.event.Event(widget.MOUSEBUTTONUP,   { "pos": (50, 660) }))

        else:
            self.disableTimer()
        
        # increment statemachine
        self.quickstart_state += 1

    
    def ok (self, trigger, event):
        """Callback triggered when the user clicks the 'Ok' button. Applies the changes after
        verifying the given data, and closes the dialog. """

        # store our name, if we got anything
        if self.user.getText () == "":
            # nothing there, use the login name
            scenario.local_player_name = self.use_this_name
        else:
            scenario.local_player_name = self.user.getText ()
        
        # we're accepting the dialog
        self.state = ACCEPTED
        
        return widget.DONE

   
                
    def cancel (self, trigger, event):
        """Callback triggered when the user clicks the 'Cancel' button. Simply closes the dialog and
        returns to the main dialog, ignoring any changes."""
        # we're cancelling the dialog
        self.state = REJECTED
        
        return widget.DONE


    def toggleFullscreen (self, trigger, event):
        """Toggles the fullscreen mode."""
        # toggle fullscreen mode on/off
        pygame.display.toggle_fullscreen ()

        # force a repaint as checkboxes leave stuff behind
        self.wm.paint(1)
        

    def toggleServer (self, trigger, event):
        """Callback triggered when one of the client/server checkboxes are clicked. This method
        makes sure the  other checkboxes are unchecked."""
        # who triggered the event?
        if trigger == self.server:
            # disable the client and ai buttons
            self.client.setChecked ( 0 )
            self.computer.setChecked ( 0 )

            # running as server
            scenario.run_as_server = 1

        elif trigger == self.client:
            # disable the server and ai buttons
            self.server.setChecked ( 0 )
            self.computer.setChecked ( 0 )

            # running as client
            scenario.run_as_server = 0

        else:
            # disable the server and ai buttons
            self.server.setChecked ( 0 )
            self.client.setChecked ( 0 )

            # running as server
            scenario.run_as_server = 1
            
        # force a repaint as checkboxes leave stuff behind
        self.wm.paint(1)


    def launchAI (self):
        """This method returns the status of the 'AI' checkbox, i.e 1 if it is checked and 0 if
        not. It does not launch the AI!"""
        return self.computer.isChecked ()
    

    def credits (self, trigger, event):
        """Callback triggered when the user clicks the 'Credits' button. Executes the dialog that
        shows credits information. Then returns to this dialog."""

        # just create a dialog and run it
        Credits ().run ()
        
        # repaint the stuff if needed
        self.wm.paint (force=1, clear=1)


    def lounge (self, trigger, event):
        """Callback triggered when the user clicks the 'Lounge' button. Executes the dialog that
        contains the lounge date. Then returns to this dialog."""

        # just create a dialog and run it
        dialog = LoungeSetup ()

        if dialog.run () != ACCEPTED:
            # the dialog was cancelled or failed, do nothing
            self.wm.paint (force=1, clear=1)
            return

        # get the connection to the lounge
        connection = dialog.getConnection ()
        
        # accepted, so run the lounge
        Lounge ( connection, self.user.getText () ).run ()
        
        # repaint the stuff if needed
        self.wm.paint (force=1, clear=1)


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
