###############################################################################################
# $Id: immediate.py,v 1.2 2002/01/24 13:41:37 chakie Exp $
###############################################################################################

import sys

class Immediate:
    """
    This class is a baseclass for all available types of immediate updates the clients may need to
    send to each other. An update differs from a plan in that it should be updated immediately when
    received, and it has usually nothing to do with any units or anything they do. The immediate
    updates are meant for internal management of game data.

    If the server needs to send immediate updates to both player it will do so by sending on the
    socket to the remote player (as normal) and by adding a line to a cache in the 'Connection'
    class. 

    All other immediate updates classes should subclass this class and implement the abstract
    methods.  
    
    Every updated contains a name which is a short lowercase string that identifies it. The name is
    unique only among classes, not instances. The name can be used to recognize the update when in
    deep debug mode.
    """
    
    def __init__ (self, name):
        """Initializes the instance. Stores the name."""
        # store the name
        self.name = name


    def getName (self):
        """Returns the name of the immediate update. This is a string that can be used to identify
        the class."""
        return self.name


    def extract (self, parameters):
        """Extracts all data from the data coming from the network. This method is used whenever
        the instance has been written to the socket and an instance needs to be recreated from textual
        data.  This method should be overridden to perform whatever is needed. The 'parameters' is a
        list of all the parameters passed with the packet."""
        pass
    

    def execute (self):
        """This optional method can perform something that is specific to the immediate update. If
        the update has nothing to do it need not override this default implementation, as it does
        nothing."""
        # do nada
        pass
    
        
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
