##############################################################################
# $Id: connection_poller_thread.py,v 1.7 2002/09/05 17:25:08 msa Exp $
##############################################################################

from connection import Connection
from threading import *
from select import select
import pygame
from pygame.locals import *

import properties
import scenario
import constants

class ConnectionPollerThread ( Thread ):
    """This class implements a thread that waits for data on a specific
    socket connection. It sends a USEREVENT with a code and
    a connection parameter. """
    
    def __init__(self, connection):
        """Initializes the thread. connection is the Connection that
        it will listen to"""
        Thread.__init__(self)
        self.connection = connection

        # allow the event we will use for notification
        pygame.event.set_allowed ( USEREVENT )


    def run (self):
        """Starts and runs the thread"""

        print "Starting connection poller thread"

        while 1:
            # We block with a timeout value
            # so we can detect when the game has ended
            try:
                incoming, out, exceptional = select ( [ self.connection ], [], [], 1.000 )
            except ValueError:
                # hmm, seems the other party has died?
                scenario.playing       = constants.GAME_ENDED
                scenario.end_game_type = constants.OPPONENT_CRASH

                # terminate ourselves
                break
            
            # Should we quit?
            if scenario.playing == constants.GAME_ENDED:
                break
            
            # Did we get anything?
            if len(incoming) == 0:
                continue

            # Create a new event
            #print "Posting event"
            try:
                e = pygame.event.Event(pygame.USEREVENT, { "code": properties.USEREVENT_DATA_FROM_SOCKET,
                                                           "connection": self.connection })
                # Send it to the main event loop
                pygame.event.post(e)
            except e:
                # Something broke badly, exit
                print "Breaking out, got exception"
                print e
                break

            # Just in case, also gives time for the main loop
            # to read the packets
            pygame.time.delay(10)

        print "Stopping connection poller thread"

