###############################################################################################
# $Id: setup.py,v 1.8 2001/09/13 10:41:34 chakie Exp $
###############################################################################################

import sys, string
from socket import *
import connection
import scenario
import properties

# a reader and writer thread
reader = None
writer = None

# define a number of error constants
NO_ERROR                 = (0, "all ok" )
SCENARIO_ERROR_SCENARIO  = (1, "players have different scenarios" )
SCENARIO_ERROR_SIDE      = (2, "players have the same side" )
NETWORK_ERROR            = (3, "network error" )
NETWORK_ERROR_SCENARIO_W = (4, "network error sending scenario name" )
NETWORK_ERROR_PLAYER_W   = (5, "network error sending player name" )
NETWORK_ERROR_SIDE_W     = (6, "network error sending side" )
NETWORK_ERROR_SCENARIO_R = (7, "network error receiving scenario name" )
NETWORK_ERROR_PLAYER_R   = (8, "network error receiving player name" )
NETWORK_ERROR_SIDE_R     = (9, "network error receiving side" )


def sendParameters (playername, scenarioname, side):
    """
    This method is used to setup the player name, scenario name and side and send it to the remote
    player. This is done using the commands 'playername', 'scenarioname' and 'side'. It simply
    writes the data to the established connection and waits for the data from the other player to be
    received. Both players do this operation. If the received scenario name differs from the one we
    the players have selected different scenarios.

    The return value is always a tuple '(code,text)' where code is 0 if all is ok and another value
    otherwise. The 'text' part can be shown to the user."""

    # send the scenario name
    if not scenario.connection.send ( "scenarioname " + scenarioname + "\n" ):
        print "sendParameters: error sending scenario"
        return NETWORK_ERROR_SCENARIO_WR

    # send the side
    if not scenario.connection.send ( "side " + side + "\n" ):
        print "sendParameters: error sending side"
        return NETWORK_ERROR_SIDE_W

    # send the player name
    if not scenario.connection.send ( "playername " + playername + "\n" ):
        print "sendParameters: error sending player"
        return NETWORK_ERROR_PLAYER_W

    # get two lines of data if one is available. We wait for max 10 minutes
    remotescenario = scenario.connection.readLine ( timeout = 600 )
    remoteside     = scenario.connection.readLine ( timeout = 10 )
    remoteplayer   = scenario.connection.readLine ( timeout = 10 )

    # did we get a remote player?
    if not remoteplayer:
        print "sendParameters: error or timeout receiving player"
        return NETWORK_ERROR_PLAYER_R

    # did we get a remote side?
    if not remoteside:
        print "sendParameters: error or timeout receiving side"
        return NETWORK_ERROR_SIDE_R

    # did we get a remote scenario?
    if not remotescenario:
        print "sendParameters: error or timeout receiving scenario"
        return NETWORK_ERROR
    
    # store the remote player name
    scenario.remote_player_name = string.join ( remoteplayer.split ()[1:] )

    # split and reassemble the scenario name and side by leaving out the first part,
    # the 'scenarioname' and 'side'
    remotescenario = string.join ( remotescenario.split ()[1:] )
    remoteside = string.join ( remoteside.split ()[1:] )
   
    # is the scenario ok
    if remotescenario != scenarioname:
        # scenario differs
        return SCENARIO_ERROR_SCENARIO

    # is the side ok
    if remoteside == side:
        # side is equeal
        return SCENARIO_ERROR_SIDE

    # all is ok
    return NO_ERROR

 
def initServer (host, port):
    """Initializes the game to run as a server. A server in this context means just the application
    that will wait for the otehr party to connect. Once the connection is established the server
    works just like to other peer/client."""
    
    # create the socket
    try:
        s = socket (AF_INET, SOCK_STREAM)

        # allow bind()ing while a previous socket is still in TIME_WAIT.
        s.setsockopt ( SOL_SOCKET, SO_REUSEADDR, 1 )

        # bind our socket for incoming requests
        s.bind ( ( host, port ) )
    except:
        print "error creating socket: " + `sys.exc_type`
        return None

    try:
        # listen for clients
        s.listen ( 1 )
        
        # accept anything that arrives
        newSocket, addr = s.accept ()

        # return the new socket
        return newSocket

    except:
        print "error listening on socket: " + `sys.exc_type`
        return None
    


def initClient (host, port):
    """Initializes the application to work as a client. A client will try to connect to the other
    party that should of course run as a server."""
    
    try:
        # create the socket
        newSocket = socket (AF_INET, SOCK_STREAM)
        
        # connect to the remote system
        newSocket.connect ( ( host, port  ) )

        # return the new and connected socket
        return newSocket
    
    except:
        print "error creating socket: " + `sys.exc_value`
        return None
       

def init ():
    """Initializes the network connection to the remote peer. Returns None if something failed, and
    another value otherwise."""

    # should we run as a server or client?
    if scenario.act_as_server:
        # we're the server
        print "init(): running as server"
        socket = initServer ( 'localhost', properties.network_port )

    else:
        # we're the client
        print "init(): running as client"
        socket = initClient ( 'localhost', properties.network_port )

    # did we get anything?
    if not socket:
        print "Failed to initialize network connection!"
        print "Maybe the socket is busy with another instance of the game?"
        return NETWORK_ERROR
    
    print "init(): network connection established ok"

    # store the connection
    scenario.connection = connection.Connection (socket)

    # all is ok
    return NO_ERROR


    
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
