###############################################################################################
# $Id: rotate.py,v 1.9 2002/09/08 12:15:11 chakie Exp $
###############################################################################################

from plan                import Plan
from mode.mode_factory   import createMode


## def create ():
##     """This function is a factory for creating instances of the class 'Rotate'. Use this when you need
##     to create instances. The new instance is returned."""
##     # create a new instance
##     rotate = Rotate ()

##     # parse out the parameters
##     #rotate.extract ( parameters )

##     # return the new instance
##     return rotate


class Rotate (Plan):
    """
    This class implements the packet 'rotate'. This plan means that the unit has been ordered to
    rotate to a new facing. The server will based on the data calculate when and how the 
    rotation takes place. The unit rotates without altering state anyhow. A unit in column mode
    rotates in column mode etc.

    Parameters:

    o the numeric id of the unit.
    o the new position the unit should be.
    """

    def __init__ (self, oldmode='invalid', unitid=-1, x=-1, y=-1):
        """Initializes the instance."""
        # call superclass constructor, the turn may not be valid here yet
        Plan.__init__ (self, "rotate", unitid, oldmode )

        # store passed data
        self.unitid  = unitid
        self.targetx = x
        self.targety = y
       
        # a nice text for the label
        self.labeltext = "rotate to (%d,%d)" % ( x, y )


    def extract (self, parameters):
        """Extracts the data for the rotation and stores in local variables. This method is used
        when the command has been sent over the network."""
        
        # parse out the data
        self.id       = int ( parameters[0] )
        self.oldmode  = createMode ( parameters[1] )
        self.delay    = int ( parameters[2] )
        self.unitid   = int ( parameters[3] )
        self.targetx  = int ( parameters[4] )
        self.targety  = int ( parameters[5] )


    def getTarget (self):
        """Returns the target position the unit should rotate to face as an (x,y) tuple."""
        return ( self.targetx, self.targety )

 
    def toString (self):
        """Returns a string representation of the command, suitable for sending over a socket."""
        # create a string and return
        return "%s %d %s %d %d %d %d\n" % ( self.getName (), self.id, self.oldmode.getName(), self.delay,
                                            self.unitid, self.targetx, self.targety )


    def __str__ (self):
        """Convenience wrapper for toString() suitable for using when debugging and printing the
        command to the screen. Will just call toString()."""
        return self.toString ()


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
