###############################################################################################
# $Id: wait.py,v 1.9 2002/09/08 12:15:11 chakie Exp $
###############################################################################################

import scenario
import properties
from   plan    import Plan
from mode.mode_factory       import createMode


## def create ():
##     """This function is a factory for creating instances of the class 'Wait'. Use this when you need
##     to create instances. The new instance is returned."""
##     # create a new instance
##     wait = Wait ()

##     # return the new instance
##     return wait


class Wait (Plan):
    """
    This class implements the plan 'wait'. This plan is used to make a unit wait for a certain
    number of minutes. When a unit waits it does nothing but keep the current mode and possibly
    defend itself against attacks. When the 'delay' has elapsed the unit proceeds with the next
    plan. 

    Parameters:

    o the numeric id of the unit.
    o the number of minutes the unit should wait.
    
    """

    def __init__ (self, oldmode='invalid', delay=-1, unitid = -1):
        """Initializes the instance."""
        # call superclass constructor, the turn may not be valid here yet
        Plan.__init__ (self, "wait", unitid, oldmode)

        # store passed data
        self.unitid = unitid

        # override the delay and make it steps instead of minutes
        self.delay = delay * ( 60 / properties.seconds_per_step )
        
        # a nice text for the label
        self.labeltext = "wait %d min" % delay


    def extract (self, parameters):
        """Extracts the data for the plan and stores in local variables. This method is used
        when the command has been sent over the network."""
        
        # parse out the data
        self.id       = int ( parameters[0] )
        self.oldmode = createMode ( parameters[1] )
        self.delay   = int ( parameters[2] )
        self.unitid  = int ( parameters[3] )

  
    def toString (self):
        """Returns a string representation of the command, suitable for sending over a socket."""
        # create a string and return
        return "%s %d %s %d %d\n" % ( self.getName (), self.id, self.oldmode.getName(), self.delay,
                                   self.unitid )


    def __str__ (self):
        """Convenience wrapper for toString() suitable for using when debugging and printing the
        command to the screen. Will just call toString()."""
        return self.toString ()


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
