###############################################################################################
# $Id: action_layer.py,v 1.6 2002/09/02 07:50:30 chakie Exp $
###############################################################################################

import sys
import pygame
from pygame.locals import *

from floating_window_layer import FloatingWindowLayer
import scenario
import properties


class ActionLayer(FloatingWindowLayer):
    """
    This class defines a layer that contains the action movie controls. It's a small dialog that
    contains four buttons that lets the player choose what to do while playing up the movie. the
    player can click on play, pause, forward, backward and beginning buttons.

    The state Action controls this layer.

    This layer is a floating window layer, which means it can be dragged around the map.
    """

    def __init__ (self, name):
        "Initializes the layer."
        # call superclass constructor
        FloatingWindowLayer.__init__ ( self, name )

        # load all icons
        self.play        =  pygame.image.load ( properties.path_dialogs + 'butt-play-moff.png').convert ()
        self.tostart     =  pygame.image.load ( properties.path_dialogs + 'butt-tostart-moff.png').convert ()
        self.pause       =  pygame.image.load ( properties.path_dialogs + 'butt-pause-moff.png').convert ()
        self.fastforward =  pygame.image.load ( properties.path_dialogs + 'butt-fastforward-moff.png').convert ()
        self.rewind      =  pygame.image.load ( properties.path_dialogs + 'butt-rewind-moff.png').convert ()

        # get width of the drawn border
        borderw, borderh = self.getBorderWidth ()

        # and the icon dimensions. we assum they're all of similar size
        self.iconwidth  = self.play.get_width ()
        self.iconheight = self.play.get_height ()

        # some spacing between icons and the border
        self.spacing = 5
        
        # set default dimensions. we want to take the border into account and then also leave a few
        # pixels space before the edges. makes it a bit more "airy". 
        self.x = scenario.sdl.getWidth () - borderw - 2 * self.iconwidth  - 3 * self.spacing - 5
        self.y = scenario.sdl.getHeight () - borderh - 2 * self.iconheight - 3 * self.spacing - 5

        # the area must fit 4 icons in a 2x2 layout
        self.width  = 2 * self.iconwidth  + 3 * self.spacing
        self.height = 2 * self.iconheight + 3 * self.spacing

        # no clicked button yet
        self.clicked = None

        # we're paused by default
        self.playing = 0
        

    def handleContentsClick (self, x, y):
        """This callback is activated if the player clicks within the contents area, ie. inside the
        borders. This callback will see if any of the buttons was pressed, and if so, do the
        nevessary action."""

        # no clicked button yet
        self.clicked = None

        # transform the coordinates to be inside the contents area
        x = x - self.x
        y = y - self.y

        # convenience variables
        basex = self.spacing
        basey = self.spacing
        sp = self.spacing
        iw = self.iconwidth
        ih = self.iconheight

        # was play clicked?
        if basex <= x <= basex + iw and basey <= y <= basey + ih:
            # play clicked
            self.clicked = "tostart"
            #print "ActionLayer.handleContentsClick: play clicked"
            
        elif basex + iw + sp <= x <= basex + 2 * iw + sp and basey <= y <= basey + ih:
            # pause clicked
            self.clicked = "playpause"
            #print "ActionLayer.handleContentsClick: pause clicked"

        elif basex <= x <= basex + iw and basey + sp + ih <= y <= basey + 2 * ih + sp:
            # rewind clicked
            self.clicked = "rewind"
            #print "ActionLayer.handleContentsClick: rewind clicked"

        elif basex + iw + sp <= x <= basex + 2 * iw + sp and basey + sp + ih <= y <= basey + 2 * ih + sp:
            # forward clicked
            self.clicked = "forward"
            #print "ActionLayer.handleContentsClick: forward clicked"

        # we were handled all right
        return 1
    

    def getClickedButton (self):
        """Returns a strin representing the button that was clicked the last time the ocntents were
        clicked. Returns None if no button was clicked or one of the strings:

        o play
        o pause
        o forward
        o rewind
        """
        return self.clicked


    def setPlayingMode (self):
        """Sets the layer in a playing mode, i.e. shows the play button and not the pause button."""
        self.playing = 1
        

    def setPausedMode (self):
        """Sets the layer in a pause mode, i.e. shows the pause button and not the play button."""
        self.playing = 0
        

    def paint (self, offsetx, offsety, dirtyrect=None):
        """Paints the layer. Blits out the surfaces for the minimap surrounded by a border."""
        # are we minimized?
        if self.isMinimized ():
            # yes, paint just the minimized layer, no content, and then go away
            self.paintBorderMinimized ( self.x, self.y, self.width )
            return
 
        # fill the background with black so that we have something to paint on
        scenario.sdl.fill ( (0,0,0), (self.x - 1 , self.y - 1, self.width + 2, self.height + 2 ) )

        # paint the border first
        self.paintBorder ( self.x, self.y, self.width, self.height )

        # get the upper left corner of the contents area. saves some typing. this includes the
        # spacing above and to the left of the icons
        x = self.x + self.spacing
        y = self.y + self.spacing
        
        # blit out the buttons
        scenario.sdl.blit ( self.tostart,     ( x, y ) )
        scenario.sdl.blit ( self.fastforward, ( x + self.iconwidth + self.spacing,
                                                y + self.iconheight + self.spacing ) )
        scenario.sdl.blit ( self.rewind,      ( x, y + self.iconheight + self.spacing ) )


        # and the final upper right button. we use a different button depending on what 'mode' we're
        # in. if we're playing we use the play button and if paused we use the pause button
        if self.playing:
            # we're playing
            scenario.sdl.blit ( self.pause,  ( x + self.iconwidth + self.spacing, y ) )
        else:
            # we're paused
            scenario.sdl.blit ( self.play, ( x + self.iconwidth + self.spacing, y ) )
        
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
