###############################################################################################
# $Id: server_main_loop.py,v 1.13 2001/11/13 15:10:12 chakie Exp $
###############################################################################################

import sys
import scenario
import pygame
import server.properties
import protocol.packethandler
from protocol.plan.new_turn   import NewTurn
from protocol.plan.plan       import Plan
from select                   import select
from player                   import Player
from net.connection           import Connection


#################################################################################################
def waitForData (clients):
    """Waits for data on the connections to the clients. Returns a list with the client connections
    which have data to be read . Will wait forever, so this function should not be called unless
    there *will* come some data back.
    """

    connections = []

    # loop over all clients
    for client in clients:
        # add the connection 
        connections.append ( client.getConnection () )
    
    # perform the select without waiting. We're only interested in incoming events
    incoming, out, execptional = select ( connections, [], [], 0.0 )
  
    # now we have the connections, but we want the clients, so check them both
    ready = []

    # loop over all clients
    for client in clients:
        # is this client in the list of clients with data to be read?
        if client.getConnection () in incoming:
            # yep, add it
            ready.append ( client )
    
    # return the list of ready clients
    return ready


#################################################################################################
def advanceTurn ():
    """Checks wether the current turn has ended. This is determined by checking the current elapsed
    time of the system, and wether it has increased more than the duration of one turn (i.e. one
    second) then the current turn is updated."""
    
    # cache the time locally
    gametime = scenario.time
    
    # do we have a new turn?
    if gametime.hasRealtimeAdvanced ():
        # DEBUGGING
        if scenario.events.has_key ( gametime.getTurn () ):
            # get the commands
            commands = scenario.events [ gametime.getTurn () ]
            print "*** still events left: %d ***" % len (commands)
            print "*** ", commands 

        # yep, we do have a new turn, update the second
        gametime.updateRealtimeSecond ()
        
        # advance the turn
        gametime.advanceTurn ()

        print "*" * 70
        print "advanceTurn: turn: %d" % gametime.getTurn ()

        # send out a plan to clients so that they know a new turn has started
        plan = NewTurn ( gametime.getTurn () - server.properties.engine_keepahead_time)
        scenario.engine.getIncomingQueue ().enqueue ( plan )

    else:
        # mikee thinks this can't happen
        raise "LogicError", "real %d game %d " % (pygame.time.get_ticks(), gametime.realtime_second)


#################################################################################################
def applyEvents ():
    """ applies all events for this turn."""
    
    # optimizations for speed. Cache locally
    turn   = scenario.time.getTurn ()
    events = scenario.events 

    # no commands for this turn an no immediate either
    commands = []
    immediate = []

    # do we have any commands for this turn?
    if events.has_key ( turn ):
        # get the commands
        commands = events [ turn ]

        # delete them
        del events [ turn ]

    # do we have any immediate commands?
    if events.has_key ( -1 ):
        # get the commands
        immediate = events [ -1 ]

        # delete them
        del events [ -1 ]

    # loop over them all
    for command in commands + immediate:
        # update the command now from the server's point of view
        print "applyEvents: at %d applying command: %s" % (turn, command.getName () )
        command.applyServer ()

 

#################################################################################################
def updateEngine (clients):
    """Lets the main logic engine update the internal state of the game. It will run events from the
    incoming event queue and add processed events to the engine's outgoing queue. The outgoing
    events are then dispatched to both players. This function keeps the game 'ticking'."""

    # update the engine
    scenario.engine.update ()

    # get the queue
    queue = scenario.engine.getOutgoingQueue ()
   
    # loop while there are items in the queue that need processing
    while not queue.isEmpty ():
        # get the front element
        command = queue.dequeue ()

        # is this a command or really a plan?
        if not isinstance ( command, Plan ):
            # a normal command, add to the local listhash of commands into the correct slot
            scenario.events.add ( command.getTurn (), command )

        #print "updateEngine: adding & sending %s" % command
        
        # serialize out it for the remote players too
        for client in clients:
            print "updateEngine: adding & sending %s" % command.toString ().strip ()
            client.getConnection ().send ( command.toString () )


#################################################################################################
def terminate ():
    """ """
    print "\nmainLoop: all clients disconnected, terminating server.\n"
    # quit pygame
    pygame.quit ()
    
    # exit with everything ok
    sys.exit ( 0 )

    
#################################################################################################
def mainLoop (clients):
    """This function is the main loop of the entire server. It is called when the players have
    connected and initialized properly. It reads commands from the players, processes them and sends
    responses back. Also runs the main engine. The passed parameter 'clients' is a list with two
    instances of Player.
    """

    print "mainLoop: entering main loop"

    # synchronize game time and real time:
    scenario.time.updateRealtimeSecond()
    
    # loop forever
    while 1:

        # loop over the clients that are ready
        for client in clients:

            # loop and read all available data from this client
            while 1:
                # attempt to read a line using its connection
                try:
                    line = client.getConnection ().readLine ( 0.0 )

                    # did we get anything?
                    if line:
                        # create a packet and add it to the incoming queue
                        protocol.packethandler.make ( line, mode=protocol.packethandler.SERVER )

                    else:
                        # the line was empty, nothing more to read from the client
                        break
                    
                except:
                    # oops, some error, remove the client
                    clients.remove ( client )
                    print "mainLoop: client disconnected, clients left:", len (clients)
                    # print line

                    # do we still have any clients left?
                    if len ( clients ) == 0:
                        # no clients left, terminate the server
                        terminate ()
                   
                    # go on with next client 
                    break
                
        # update the engine
        updateEngine ( clients )
     
        # apply outstanding events
        applyEvents()

        # wait until next turn (plus 20 milliseconds just in case)
        try:
            pygame.time.delay ( scenario.time.getRealtimeUntilAdvance() + 20 )

        except "GameAdvanceLate", lag:
                print 'Server ahead of realtime, waiting.', lag
                pygame.time.delay ( int(lag) + 10 )

        except "GameAdvanceEarly":
                raise "LogicError", "skipped a turn??"


        # ok, advance to next turn!
        advanceTurn ()
        
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
