###############################################################################################
# $Id: calculate_action.py,v 1.12 2002/09/05 16:58:49 msa Exp $
###############################################################################################

import sys
import scenario
import properties
import pygame
import pygame.event
from pygame.locals       import *

from organization import REBEL, UNION

import state
import action

import immediate.end_action

class CalculateAction (state.State):
    """
    This class is a state that is used by the server when it is calculating action data. This state
    has as its sole purpose to be a 'shell' for that action calculation. Only the server will ever
    use this state.

    The tasks are:

    * calculate the action
    * show a dialog so that the player knows that something is happening
    * provide some form of input?
    * send all action to the remote client and make it available to the local client

    When this state is finished (data calculate and sent) the action state is started.
    """

    def __init__ (self):
        """Initializes the instance. Sets default values for all needed member."""

        # call superclass constructor
        state.State.__init__ (self)

        # set default cursor
        self.setDefaultCursor ()
       
        # set defaults
        self.name = "calculate_action"

        # find the layer that we use for information
        self.layer = scenario.playfield.getLayer ( "calculate_action" )

        # and make it visible
        scenario.playfield.setVisible ( self.layer )

        # Call us soon so we can do something...
        self.calculated = 0
        self.callMeSoon ()
        

    def handleEvent (self, event):
        """This event handler is only useful the first time it is called.
        This is really a little kludge, see comment in the constructor above."""

        # have we already calculated the action?
        if self.calculated == 0:
            self.calculated = 1

            # update the engine 
            scenario.engine.update ()

            # send all action data to the other client
            self.sendActionData ()

            # now return a new state for viewing the action
            return self.viewAction ()

        # Otherwise, perform the default handling
        return state.State.handleEvent(self, event)
       
        
    def viewAction (self):
        """Ends the state and hides the dialog and repaints the playfield. Starts viewing the
        action, which much have been calculated and sent by now."""

        # find the layer and hide it
        scenario.playfield.setVisible ( self.layer, 0 )

        # start the action
        return action.Action ()


    def sendActionData (self):
        """Sends all data that the server has calculated to the remote client. This is done as the
        last step in this state. Only the server player gets sent the data. Last an immediate update
        that tells the remote player to start displaying is sent."""
  
        print "CalculateAction.sendActionData: sending action data"

        # get the id of the remote player
        remote = [UNION, REBEL][scenario.local_player_id]

        # loop over all units. this will clear all the plans for remote players. why do this then?
        # well, the next turn the remote player will send the plans again, and if these are still
        # here the unit will have duplicate plan
        for unit in scenario.info.units.values ():
            # is this a remote player unit?
            if unit.getOwner () == remote:
                # a remote unit, clear its plans
                unit.setPlans ( [] )

                #print "CalculateAction.sendActionData: #+#+#+# clearing for", unit
           
        # loop over all data that has been accumulated during the action calculations
        for act in scenario.engine.getAction ():
            # we need to send the action over the socket, and also directly put it on the events hash
            if not scenario.events.has_key ( act.getStep () ):
                # create an empty list there
                scenario.events [ act.getStep () ] = []

            # now append it
            scenario.events [ act.getStep () ].append ( act )

            # send it too
            scenario.connection.send ( act.toString () )

            scenario.dispatcher.emit("send_action_data", {})

            
        # now let the client know we're done sending, let the action start. this tells the client
        # that all action data has now been sent off and that action visualization can proceed
        
        # create a new immediate update and send to the client
        update = immediate.end_action.EndAction ()
        
        # send off the update
        scenario.connection.send ( update.toString () )

        print "CalculateAction.sendActionData: sent end action:", update


        
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
