###############################################################################################
# $Id: check_los.py,v 1.21 2002/09/07 13:26:27 msa Exp $
###############################################################################################

import sys
import scenario
import properties
import pygame
import map
from pygame.locals import *

import state
import own_unit

class CheckLos (state.State):
    """
    This class is a state that takes care of letting the user check line of sight (los) for the
    current unit to various positions on the map. As long as the player left-clicks a position on
    the map LOS to that position is checked for the current unit. All calculations are done
    internally on the client, nothing is sent/received to/from the server.

    Pressing 'Escape' terminates the LOS-checking.
    """

    # define a shared base cursor
    cursor = None

    def __init__ (self):
        """Initializes the instance. Sets default values for all needed members."""

        # call superclass constructor
        state.State.__init__ (self)

        # do we have a cursor already loaded? 
        if not CheckLos.cursor:
            # nope, so load it. First get the filenames
            datafile = properties.state_check_los_cursor_data
            maskfile = properties.state_check_los_cursor_mask

            # now load it
            CheckLos.cursor = pygame.cursors.load_xbm ( datafile, maskfile )

        # set our own cursor cursor
        pygame.mouse.set_cursor ( *CheckLos.cursor )
         
        # set defaults
        self.name = "check_los"
        
        # set the keymap too
        self.keymap [ (K_ESCAPE, KMOD_NONE) ] = self.cancel

        # define the help text too
        self.helptext = [ "Line of sight test",
                          " ",
                          "arrow keys - scroll map",
                          "F1 - show this help text",
                          "F10 - toggle fullscreen mode",
                          "F12 - save a screenshot",
                          "esc - cancel the los test" ]
        

    def cancel (self):
        """Callback triggered when the user presses the 'escape' key. Cancels the moving and makes a
        OwnUnit state active again."""
        # return a new state
        return own_unit.OwnUnit ()

        
    def handleLeftMousePressed (self, event):
        """Method for handling a mouse pressed. Checks wether the mouse was clicked in the map or in
        the panel. Only clicks in the map will do anything useful, all other clicks are
        ignored.

        If the click is in the map then LOS to that position is checked for the given unit.
        """
        # get event position
        x, y = event.pos
      
        # get unit
        unit = self.getSelectedUnit ()

        # the click is on the main playfield, so get the clicked coordinate
        globalx, globaly = self.toGlobal ( (x, y) )

        # get unit position
        unitx, unity = unit.getPosition ()

        print "CheckLos.handleLeftMousePressed: los from %d,%d to %d,%d" % (unitx,unity,globalx,globaly)
        print "  Visibility is %d" % scenario.map.losValue(unit.getPosition(),
                                                           (globalx,globaly),unit.sightRange)

        hexCoord = scenario.map.pointToHex2((globalx,globaly))
        hexCenter = scenario.map.hexToPoint(hexCoord)
        
        print "  Hex of point (%d,%d) is (%d,%d) " % (globalx, globaly, hexCoord[0], hexCoord[1])
        print "  Terrain type at point is %s " % scenario.map.getTerrain((globalx,globaly)).getType()
        print "  Height of hex is %d" % scenario.map.hexHeight( hexCoord[0], hexCoord[1] )

        # do not change state, remain in this until escaped
        return None
 
    
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
